import m from "mithril";
import { TextField } from "../../../common/gui/base/TextField.js";
import { createAlarmIntervalItems, createCustomRepeatRuleUnitValues, humanDescriptionForAlarmInterval } from "./CalendarGuiUtils.js";
import { lang } from "../../../common/misc/LanguageViewModel.js";
import { IconButton } from "../../../common/gui/base/IconButton.js";
import { attachDropdown } from "../../../common/gui/base/Dropdown.js";
import { AlarmIntervalUnit } from "../../../common/calendar/date/CalendarUtils.js";
import { Dialog } from "../../../common/gui/base/Dialog.js";
import { DropDownSelector } from "../../../common/gui/base/DropDownSelector.js";
import { deepEqual } from "@tutao/tutanota-utils";
export class RemindersEditor {
    view(vnode) {
        const { addAlarm, removeAlarm, alarms } = vnode.attrs;
        const addNewAlarm = (newAlarm) => {
            const hasAlarm = alarms.find((alarm) => deepEqual(alarm, newAlarm));
            if (hasAlarm)
                return;
            addAlarm(newAlarm);
        };
        const textFieldAttrs = alarms.map((a) => ({
            value: humanDescriptionForAlarmInterval(a, lang.languageTag),
            label: "emptyString_msg",
            isReadOnly: true,
            injectionsRight: () => m(IconButton, {
                title: "delete_action",
                icon: "Cancel" /* Icons.Cancel */,
                click: () => removeAlarm(a),
            }),
        }));
        textFieldAttrs.push({
            value: lang.get("add_action"),
            label: "emptyString_msg",
            isReadOnly: true,
            injectionsRight: () => m(IconButton, attachDropdown({
                mainButtonAttrs: {
                    title: "add_action",
                    icon: "Add" /* Icons.Add */,
                },
                childAttrs: () => [
                    ...createAlarmIntervalItems(lang.languageTag).map((i) => ({
                        label: () => i.name,
                        click: () => addNewAlarm(i.value),
                    })),
                    {
                        label: () => lang.get("calendarReminderIntervalDropdownCustomItem_label"),
                        click: () => {
                            this.showCustomReminderIntervalDialog((value, unit) => {
                                addNewAlarm({
                                    value,
                                    unit,
                                });
                            });
                        },
                    },
                ],
            })),
        });
        textFieldAttrs[0].label = vnode.attrs.label;
        return m(".flex.col.flex-half.pl-s", textFieldAttrs.map((a) => m(TextField, a)));
    }
    showCustomReminderIntervalDialog(onAddAction) {
        let timeReminderValue = 0;
        let timeReminderUnit = AlarmIntervalUnit.MINUTE;
        Dialog.showActionDialog({
            title: () => lang.get("calendarReminderIntervalCustomDialog_title"),
            allowOkWithReturn: true,
            child: {
                view: () => {
                    const unitItems = createCustomRepeatRuleUnitValues() ?? [];
                    return m(".flex full-width pt-s", [
                        m(TextField, {
                            type: "number" /* TextFieldType.Number */,
                            min: 0,
                            label: "calendarReminderIntervalValue_label",
                            value: timeReminderValue.toString(),
                            oninput: (v) => {
                                const time = Number.parseInt(v);
                                const isEmpty = v === "";
                                if (!Number.isNaN(time) || isEmpty)
                                    timeReminderValue = isEmpty ? 0 : Math.abs(time);
                            },
                            class: "flex-half no-appearance", //Removes the up/down arrow from input number. Pressing arrow up/down key still working
                        }),
                        m(DropDownSelector, {
                            label: "emptyString_msg",
                            selectedValue: timeReminderUnit,
                            items: unitItems,
                            class: "flex-half pl-s",
                            selectionChangedHandler: (selectedValue) => (timeReminderUnit = selectedValue),
                            disabled: false,
                        }),
                    ]);
                },
            },
            okActionTextId: "add_action",
            okAction: (dialog) => {
                onAddAction(timeReminderValue, timeReminderUnit);
                dialog.close();
            },
        });
    }
}
//# sourceMappingURL=RemindersEditor.js.map