import { PluginBuild } from "esbuild";
import { FallbackOptions } from "./WasmHandler.js";
export interface Library {
    /** Name of the module, how it is imported in JS */
    name: string;
    /** Command to run to generate WASM */
    command: string;
    /** Optimization level for the JavaScript fallback */
    optimizationLevel?: string;
    /** Where to run the command */
    workingDir?: string;
    /** Environment variables to be set for compilation */
    env?: Record<string, any>;
}
/**
 * Top-level plugin options
 */
export interface PluginOptions {
    /** Output path for the webassembly files */
    output: string;
    /** List of webassembly files to be compiled and get fallback generated */
    webassemblyLibraries: Library[];
    /** Whether to generate JS fallback for WASM. If {@code true} or an object it will generate fallback code. */
    fallback?: FallbackOptions | boolean;
}
export declare function esbuildWasmLoader(options: PluginOptions): {
    name: string;
    setup(build: PluginBuild): void;
};
export declare function rollupWasmLoader(options: PluginOptions & {
    output: string;
}): {
    name: string;
    resolveId(source: string): Promise<{
        id: string;
        external: boolean;
    } | undefined>;
    resolveDynamicImport(specifier: string, importer: string): Promise<{
        external: boolean;
        id: string;
    } | undefined>;
    load(id: string): Promise<string | undefined>;
};
export { esbuildWasmLoader as default };
